@@ -9,7 +9,7 @@
   Copyright:
   (C) 2002-2003 by José Pablo Ezequiel "Pupeno" Fernández <pupeno@kde.org>
   (C) 2003-2004 by Olaf Schmidt <ojschmidt@kde.org>
-  (C) 2004 by Gary Cramblitt <garycramblitt@comcast.net>
+  (C) 2004-2005 by Gary Cramblitt <garycramblitt@comcast.net>
   -------------------
   Original author: José Pablo Ezequiel "Pupeno" Fernández
  ******************************************************************************/
@@ -22,18 +22,31 @@
  *                                                                            *
  ******************************************************************************/
 
+// C++ includes.
 #include <stdlib.h>
 
-#include <kdebug.h>
-#include <kglobal.h>
-#include <kapplication.h>
+// Qt includes.
 #include <qregexp.h>
 #include <qpair.h>
 #include <qvaluelist.h>
+#include <qdom.h>
 
+// KDE includes.
+#include <kdebug.h>
+#include <kglobal.h>
+#include <kstandarddirs.h>
+#include <kapplication.h>
+
+// KTTS includes.
+#include "talkermgr.h"
+
+// SpeechData includes.
 #include "speechdata.h"
 #include "speechdata.moc"
 
+// Set this to 1 to turn off filter support, including SBD as a plugin.
+#define NO_FILTERS 0
+
 /**
 * Constructor
 * Sets text to be stopped and warnings and messages queues to be autodelete.
@@ -51,6 +64,7 @@ SpeechData::SpeechData(){
     // Messages queue to be autodelete (thread safe)
     messages.setAutoDelete(true);
 
+    screenReaderOutput.jobNum = 0;
     screenReaderOutput.text = "";
 }
 
@@ -68,18 +82,43 @@ bool SpeechData::readConfig(){
     textPreMsg = config->readEntry("TextPreMsg");
 
     textPreSndEnabled = config->readBoolEntry("TextPreSndEnabled", false);
-    textPreSnd = config->readPathEntry("TextPreSnd");
+    textPreSnd = config->readEntry("TextPreSnd");
 
     textPostMsgEnabled = config->readBoolEntry("TextPostMsgEnabled", false);
     textPostMsg = config->readEntry("TextPostMsg");
 
     textPostSndEnabled = config->readBoolEntry("TextPostSndEnabled", false);
-    textPostSnd = config->readPathEntry("TextPostSnd");
+    textPostSnd = config->readEntry("TextPostSnd");
+    keepAudio = config->readBoolEntry("KeepAudio", false);
+    keepAudioPath = config->readEntry("KeepAudioPath", locateLocal("data", "kttsd/audio/"));
 
     // Notification (KNotify).
     notify = config->readBoolEntry("Notify", false);
+    notifyPopupsOnly = config->readBoolEntry("NotifyPopupsOnly", true);
     notifyPassivePopupsOnly = config->readBoolEntry("NotifyPassivePopupsOnly", false);
 
+    // Clear the pool of filter managers so that filters re-init themselves.
+    QIntDictIterator<PooledFilterMgr> it( m_pooledFilterMgrs );
+    for( ; it.current(); ++it )
+    {
+        PooledFilterMgr* pooledFilterMgr = it.current();
+        delete pooledFilterMgr->filterMgr;
+        delete pooledFilterMgr;
+    }
+    m_pooledFilterMgrs.clear();
+
+    // Create an initial FilterMgr for the pool to save time later.
+    PooledFilterMgr* pooledFilterMgr = new PooledFilterMgr();
+    FilterMgr* filterMgr = new FilterMgr();
+    filterMgr->init(config, "General");
+    pooledFilterMgr->filterMgr = filterMgr;
+    pooledFilterMgr->busy = false;
+    pooledFilterMgr->job = 0;
+    // Connect signals from FilterMgr.
+    connect (filterMgr, SIGNAL(filteringFinished()), this, SLOT(slotFilterMgrFinished()));
+    connect (filterMgr, SIGNAL(filteringStopped()),  this, SLOT(slotFilterMgrStopped()));
+    m_pooledFilterMgrs.insert(0, pooledFilterMgr);
+
     return true;
 }
 /**
@@ -92,6 +131,15 @@ SpeechData::~SpeechData(){
     {
         emit textRemoved(job->appId, job->jobNum);
     }
+
+    QIntDictIterator<PooledFilterMgr> it( m_pooledFilterMgrs );
+    for( ; it.current(); ++it )
+    {
+        PooledFilterMgr* pooledFilterMgr = it.current();
+        delete pooledFilterMgr->filterMgr;
+        delete pooledFilterMgr;
+    }
+
     delete config;
 }
 
@@ -146,6 +194,7 @@ bool SpeechData::screenReaderOutputReady()
 void SpeechData::enqueueWarning( const QString &warning, const QString &talker, const QCString &appId){
     // kdDebug() << "Running: SpeechData::enqueueWarning( const QString &warning )" << endl;
     mlText *temp = new mlText();
+    temp->jobNum = 0;
     temp->text = warning;
     temp->talker = talker;
     temp->appId = appId;
@@ -188,6 +237,24 @@ bool SpeechData::warningInQueue(){
 */
 void SpeechData::enqueueMessage( const QString &message, const QString &talker, const QCString& appId){
     // kdDebug() << "Running: SpeechData::enqueueMessage" << endl;
+    mlJob* job = new mlJob();
+    ++jobCounter;
+    if (jobCounter == 0) ++jobCounter;  // Overflow is OK, but don't want any 0 jobNums.
+    uint jobNum = jobCounter;
+    job->jobNum = jobNum;
+    job->talker = talker;
+    job->appId = appId;
+    job->seq = 1;
+    messages.enqueue( job );
+    // If message came from knotify, filter it, otherwise, do not filter.
+    if (appId.left(7) == "knotify")
+    {
+        job->sentences = QStringList();
+        // Do not apply Sentence Boundary Detection filters to messages.
+        startJobFiltering( job, message, true );
+    } else
+        job->sentences = message;
+    /*
     mlText *temp = new mlText();
     temp->text = message;
     temp->talker = talker;
@@ -196,6 +263,7 @@ void SpeechData::enqueueMessage( const QString &message, const QString &talker,
     messages.enqueue( temp );
     // uint count = messages.count();
     // kdDebug() << "Adding '" << temp->text << "' with talker '" << temp->talker << "' from application " << appId << " to the messages queue leaving a total of " << count << " items." << endl;
+    */
 }
 
 /**
@@ -206,8 +274,17 @@ void SpeechData::enqueueMessage( const QString &message, const QString &talker,
 */
 mlText* SpeechData::dequeueMessage(){
     // kdDebug() << "Running: SpeechData::dequeueMessage()" << endl;
-    mlText *temp = messages.dequeue();
-    // uint count = warnings.count();
+    mlJob* job = messages.dequeue();
+    waitJobFiltering(job);
+    mlText* temp = new mlText();
+    temp->jobNum = job->jobNum;
+    temp->text = job->sentences.join("");
+    temp->talker = job->talker;
+    temp->appId = job->appId;
+    temp->seq = 1;
+    delete job;
+    /* mlText *temp = messages.dequeue(); */
+    // uint count = messages.count();
     // kdDebug() << "Removing '" << temp->text << "' with talker '" << temp->talker << "' from the messages queue leaving a total of " << count << " items." << endl;
     return temp;
 }
@@ -226,17 +303,18 @@ bool SpeechData::messageInQueue(){
     return temp;
 }
 
-#if SUPPORT_SSML
 /**
 * Determines whether the given text is SSML markup.
 */
 bool SpeechData::isSsml(const QString &text)
 {
-    // TODO: This is really simple and braindead right now.  A better method might be
-    // to use a SAX parser and look for any SSML tags.  Return true on the first found.
-    return (text.contains("<speak") > 0);
+    /// This checks to see if the root tag of the text is a <speak> tag. 
+    QDomDocument ssml;
+    ssml.setContent(text, false);  // No namespace processing.
+    /// Check to see if this is SSML
+    QDomElement root = ssml.documentElement();
+    return (root.tagName() == "speak");
 }
-#endif
 
 /**
 * Parses a block of text into sentences using the application-specified regular expression
@@ -254,13 +332,11 @@ QStringList SpeechData::parseText(const QString &text, const QCString &appId /*=
 {
     // There has to be a better way
     // kdDebug() << "I'm getting: " << endl << text << " from application " << appId << endl;
-#if SUPPORT_SSML
     if (isSsml(text))
     {
         QString tempList(text);
         return tempList;
     }
-#endif
     // See if app has specified a custom sentence delimiter and use it, otherwise use default.
     QRegExp sentenceDelimiter;
     if (sentenceDelimiters.find(appId) != sentenceDelimiters.end())
@@ -283,12 +359,7 @@ QStringList SpeechData::parseText(const QString &text, const QCString &appId /*=
     temp.replace(QRegExp("\t\t+"),"\t");
     // Split into sentences.
     QStringList tempList = QStringList::split("\t", temp, false);
-/*
-    // This should be something better, like "[a-zA-Z]\. " (a regexp of course) The dot (.) is used for more than ending a sentence.
-    temp.replace('.', '\n');
-    QStringList tempList = QStringList::split('\n', temp, true);
-*/
-    
+
 //    for ( QStringList::Iterator it = tempList.begin(); it != tempList.end(); ++it ) {
 //        kdDebug() << "'" << *it << "'" << endl;
 //    }
@@ -301,22 +372,27 @@ QStringList SpeechData::parseText(const QString &text, const QCString &appId /*=
 uint SpeechData::setText( const QString &text, const QString &talker, const QCString &appId)
 {
     // kdDebug() << "Running: SpeechData::setText" << endl;
-    QStringList tempList = parseText(text, appId);
-    if (talker != NULL)
-        textTalker = talker;
-    else
-        textTalker = QString::null;
     mlJob* job = new mlJob;
-    uint jobNum = ++jobCounter;
+    ++jobCounter;
+    if (jobCounter == 0) ++jobCounter;  // Overflow is OK, but don't want any 0 jobNums.
+    uint jobNum = jobCounter;
     job->jobNum = jobNum;
     job->appId = appId;
-    job->talker = textTalker;
-    job->state = kspeech::jsQueued;
+    job->talker = talker;
+    job->state = KSpeech::jsQueued;
     job->seq = 0;
+#if NO_FILTERS
+    QStringList tempList = parseText(text, appId);
     job->sentences = tempList;
     job->partSeqNums.append(tempList.count());
     textJobs.append(job);
     emit textSet(appId, jobNum);
+#else
+    job->sentences = QStringList();
+    job->partSeqNums = QValueList<int>();
+    textJobs.append(job);
+    startJobFiltering(job, text, false);
+#endif
     return jobNum;
 }
 
@@ -336,19 +412,24 @@ uint SpeechData::setText( const QString &text, const QString &talker, const QCSt
 * @see setText.
 * @see startText.
 */
-int SpeechData::appendText(const QString &text, const uint jobNum, const QCString& appId)
+int SpeechData::appendText(const QString &text, const uint jobNum, const QCString& /*appId*/)
 {
     // kdDebug() << "Running: SpeechData::appendText" << endl;
-    QStringList tempList = parseText(text, appId);
     int newPartNum = 0;
     mlJob* job = findJobByJobNum(jobNum);
     if (job)
     {
+#if NO_FILTERS
+        QStringList tempList = parseText(text, appId);
         int sentenceCount = job->sentences.count();
         job->sentences += tempList;
         job->partSeqNums.append(sentenceCount + tempList.count());
-        newPartNum = job->partSeqNums.count();
+        newPartNum = job->partSeqNums.count() + 1;
         emit textAppended(job->appId, jobNum, newPartNum);
+#else
+        newPartNum = job->partSeqNums.count() + 1;
+        startJobFiltering(job, text, false);
+#endif
     }
     return newPartNum;
 }
@@ -447,6 +528,11 @@ QCString SpeechData::getAppIdByJobNum(const uint jobNum)
     return appId;
 }
 
+/**
+* Sets pointer to the TalkerMgr object.
+*/
+void SpeechData::setTalkerMgr(TalkerMgr* talkerMgr) { m_talkerMgr = talkerMgr; }
+
 /**
 * Remove a text job from the queue.
 * (thread safe)
@@ -464,6 +550,14 @@ void SpeechData::removeText(const uint jobNum)
     {
         removeAppId = removeJob->appId;
         removeJobNum = removeJob->jobNum;
+        // If filtering on the job, cancel it.
+        if (m_pooledFilterMgrs[removeJobNum])
+        {
+            PooledFilterMgr* pooledFilterMgr = m_pooledFilterMgrs[removeJobNum];
+            pooledFilterMgr->busy = false;
+            pooledFilterMgr->job = 0;
+            pooledFilterMgr->filterMgr->stopFiltering();
+        }
         // Delete the job.
         textJobs.removeRef(removeJob);
     }
@@ -484,6 +578,8 @@ int SpeechData::getJobPartNumFromSeq(const mlJob& job, const int seq)
     int foundPartNum = 0;
     int desiredSeq = seq;
     uint partNum = 0;
+    // Wait until all filtering has stopped for the job.
+    waitJobFiltering(&job);
     while (partNum < job.partSeqNums.count())
     {
         if (desiredSeq <= job.partSeqNums[partNum])
@@ -500,7 +596,6 @@ int SpeechData::getJobPartNumFromSeq(const mlJob& job, const int seq)
 
 /**
 * Delete expired jobs.  At most, one finished job is kept on the queue.
-* (thread safe)
 * @param finishedJobNum Job number of a job that just finished.
 * The just finished job is not deleted, but any other finished jobs are.
 * Does not change the textJobs.current() pointer.
@@ -514,7 +609,7 @@ void SpeechData::deleteExpiredJobs(const uint finishedJobNum)
     // Walk through jobs and delete any other finished jobs.
     for (mlJob* job = textJobs.first(); (job); job = textJobs.next())
     {
-        if (job->jobNum != finishedJobNum and job->state == kspeech::jsFinished)
+        if (job->jobNum != finishedJobNum && job->state == KSpeech::jsFinished)
         {
             removedJobs.append(removedJob(job->appId, job->jobNum));
             textJobs.removeRef(job);
@@ -543,7 +638,11 @@ mlJob* SpeechData::getNextSpeakableJob(const uint prevJobNum)
 {
     for (mlJob* job = textJobs.first(); (job); job = textJobs.next())
         if (job->jobNum != prevJobNum)
-            if (job->state == kspeech::jsSpeakable) return job;
+            if (job->state == KSpeech::jsSpeakable)
+            {
+                waitJobFiltering(job);
+                return job;
+            }
     return 0;
 }
 
@@ -560,7 +659,7 @@ mlJob* SpeechData::getNextSpeakableJob(const uint prevJobNum)
 */
 mlText* SpeechData::getNextSentenceText(const uint prevJobNum, const uint prevSeq)
 {
-    kdDebug() << "SpeechData::getNextSentenceText running with prevJobNum " << prevJobNum << " prevSeq " << prevSeq << endl;
+    // kdDebug() << "SpeechData::getNextSentenceText running with prevJobNum " << prevJobNum << " prevSeq " << prevSeq << endl;
     mlText* temp = 0;
     uint jobNum = prevJobNum;
     mlJob* job = 0;
@@ -579,7 +678,7 @@ mlText* SpeechData::getNextSentenceText(const uint prevJobNum, const uint prevSe
         }
         else
         {
-            if ((job->state != kspeech::jsSpeakable) and (job->state != kspeech::jsSpeaking))
+            if ((job->state != KSpeech::jsSpeakable) && (job->state != KSpeech::jsSpeaking))
             {
                 job = getNextSpeakableJob(job->jobNum);
                 if (job) seq =+ job->seq;
@@ -604,8 +703,8 @@ mlText* SpeechData::getNextSentenceText(const uint prevJobNum, const uint prevSe
         temp->talker = job->talker;
         temp->jobNum = job->jobNum;
         temp->seq = seq;
-        kdDebug() << "SpeechData::getNextSentenceText: return job number " << temp->jobNum << " seq " << temp->seq << endl;
-    } else kdDebug() << "SpeechData::getNextSentenceText: no more sentences in queue" << endl;
+        // kdDebug() << "SpeechData::getNextSentenceText: return job number " << temp->jobNum << " seq " << temp->seq << endl;
+    } // else kdDebug() << "SpeechData::getNextSentenceText: no more sentences in queue" << endl;
     return temp;
 }
 
@@ -669,8 +768,10 @@ int SpeechData::getTextCount(const uint jobNum)
     mlJob* job = findJobByJobNum(jobNum);
     int temp;
     if (job)
+    {
+        waitJobFiltering(job);
         temp = job->sentences.count();
-    else
+    } else
         temp = -1;
     return temp;
 }
@@ -726,13 +827,13 @@ int SpeechData::getTextJobState(const uint jobNum)
 * If the new state is Finished, deletes other expired jobs.
 *
 **/
-void SpeechData::setTextJobState(const uint jobNum, const kspeech::kttsdJobState state)
+void SpeechData::setTextJobState(const uint jobNum, const KSpeech::kttsdJobState state)
 {
     mlJob* job = findJobByJobNum(jobNum);
     if (job)
     {
         job->state = state;
-        if (state == kspeech::jsFinished) deleteExpiredJobs(jobNum);
+        if (state == KSpeech::jsFinished) deleteExpiredJobs(jobNum);
     }
 }
 
@@ -780,6 +881,7 @@ QByteArray SpeechData::getTextJobInfo(const uint jobNum)
     QByteArray temp;
     if (job)
     {
+        waitJobFiltering(job);
         QDataStream stream(temp, IO_WriteOnly);
         stream << job->state;
         stream << job->appId;
@@ -805,11 +907,12 @@ QString SpeechData::getTextJobSentence(const uint jobNum, const uint seq /*=1*/)
     QString temp;
     if (job)
     {
+        waitJobFiltering(job);
         temp = job->sentences[seq - 1];
     }
     return temp;
 }
-       
+
 /**
 * Change the talker for a text job.
 * @param jobNum         Job number of the text job.
@@ -820,16 +923,10 @@ QString SpeechData::getTextJobSentence(const uint jobNum, const uint seq /*=1*/)
 *                       If no plugin has been configured for the specified Talker code,
 *                       defaults to the closest matching talker.
 */
-void SpeechData::changeTextTalker(const uint jobNum, const QString &talker)
+void SpeechData::changeTextTalker(const QString &talker, uint jobNum)
 {
     mlJob* job = findJobByJobNum(jobNum);
-    if (job)
-    {
-        if (!talker.isEmpty())
-            job->talker = talker;
-        else
-            job->talker = QString::null;
-    }
+    if (job) job->talker = talker;
 }
 
 /**
@@ -868,6 +965,7 @@ int SpeechData::jumpToTextPart(const int partNum, const uint jobNum)
     mlJob* job = findJobByJobNum(jobNum);
     if (job)
     {
+        waitJobFiltering(job);
         if (partNum > 0)
         {
             newPartNum = partNum;
@@ -903,6 +1001,7 @@ uint SpeechData::moveRelTextSentence(const int n, const uint jobNum /*=0*/)
     mlJob* job = findJobByJobNum(jobNum);
     if (job)
     {
+        waitJobFiltering(job);
         int oldSeqNum = job->seq;
         newSeqNum = oldSeqNum + n;
         if (n != 0)
@@ -916,3 +1015,144 @@ uint SpeechData::moveRelTextSentence(const int n, const uint jobNum /*=0*/)
     return newSeqNum;
 }
 
+/**
+* Assigns a FilterMgr to a job and starts filtering on it.
+*/
+void SpeechData::startJobFiltering(mlJob* job, const QString& text, bool noSBD)
+{
+    // If filtering is already in progress for this job, do nothing.
+    uint jobNum = job->jobNum;
+    PooledFilterMgr* pooledFilterMgr = m_pooledFilterMgrs[jobNum];
+    if (pooledFilterMgr) return;
+    // Find an idle FilterMgr, if any.
+    QIntDictIterator<PooledFilterMgr> it( m_pooledFilterMgrs );
+    for( ; it.current(); ++it )
+    {
+        if (!it.current()->busy)
+        {
+            // Reindex the pooled FilterMgr on the new job number.
+            int oldJobNum = it.currentKey();
+            pooledFilterMgr = m_pooledFilterMgrs.take(oldJobNum);
+            m_pooledFilterMgrs.insert(jobNum, pooledFilterMgr);
+            break;
+        }
+    }
+    // Create a new FilterMgr if needed and add to pool.
+    if (!pooledFilterMgr)
+    {
+        pooledFilterMgr = new PooledFilterMgr();
+        FilterMgr* filterMgr = new FilterMgr();
+        filterMgr->init(config, "General");
+        pooledFilterMgr->filterMgr = filterMgr;
+        // Connect signals from FilterMgr.
+        connect (filterMgr, SIGNAL(filteringFinished()), this, SLOT(slotFilterMgrFinished()));
+        connect (filterMgr, SIGNAL(filteringStopped()),  this, SLOT(slotFilterMgrStopped()));
+        m_pooledFilterMgrs.insert(jobNum, pooledFilterMgr);
+    }
+    // Flag the FilterMgr as busy and set it going.
+    pooledFilterMgr->busy = true;
+    pooledFilterMgr->job = job;
+    pooledFilterMgr->filterMgr->setNoSBD( noSBD );
+    // Get TalkerCode structure of closest matching Talker.
+    pooledFilterMgr->talkerCode = m_talkerMgr->talkerToTalkerCode(job->talker);
+    // Pass Sentence Boundary regular expression (if app overrode default);
+    if (sentenceDelimiters.find(job->appId) != sentenceDelimiters.end())
+        pooledFilterMgr->filterMgr->setSbRegExp(sentenceDelimiters[job->appId]);
+    pooledFilterMgr->filterMgr->asyncConvert(text, pooledFilterMgr->talkerCode, job->appId);
+}
+
+/**
+* Waits for filtering to be completed on a job.
+* This is typically called because an app has requested job info that requires
+* filtering to be completed, such as getJobInfo.
+*/
+void SpeechData::waitJobFiltering(const mlJob* job)
+{
+#if NO_FILTERS
+    return;
+#endif
+    PooledFilterMgr* pooledFilterMgr = m_pooledFilterMgrs[job->jobNum];
+    if (!pooledFilterMgr) return;
+    if (pooledFilterMgr->busy)
+    {
+        if (!pooledFilterMgr->filterMgr->noSBD())
+            kdDebug() << "SpeechData::waitJobFiltering: Waiting for filter to finish.  Not optimium.  " <<
+                "Try waiting for textSet signal before querying for job information." << endl;
+        pooledFilterMgr->filterMgr->waitForFinished();
+        doFiltering();
+    }
+}
+
+/**
+* Processes filters by looping across the pool of FilterMgrs.
+* As each FilterMgr finishes, emits appropriate signals and flags it as no longer busy.
+*/
+void SpeechData::doFiltering()
+{
+    // kdDebug() << "SpeechData::doFiltering: Running." << endl;
+    bool again = true;
+    while (again)
+    {
+        again = false;
+        QIntDictIterator<PooledFilterMgr> it( m_pooledFilterMgrs );
+        for( ; it.current(); ++it )
+        {
+            PooledFilterMgr* pooledFilterMgr = it.current();
+            // If FilterMgr is busy, see if it is now finished.
+            if (pooledFilterMgr->busy)
+            {
+                FilterMgr* filterMgr = pooledFilterMgr->filterMgr;
+                if (filterMgr->getState() == FilterMgr::fsFinished)
+                {
+                    pooledFilterMgr->busy = false;
+                    mlJob* job = pooledFilterMgr->job;
+                    // Retrieve text from FilterMgr.
+                    QString text = filterMgr->getOutput();
+                    // kdDebug() << "SpeechData::doFiltering: text.left(500) = " << text.left(500) << endl;
+                    // kdDebug() << "SpeechData::doFiltering: filtered text: " << text << endl;
+                    filterMgr->ackFinished();
+                    // Convert the TalkerCode back into string.
+                    job->talker = pooledFilterMgr->talkerCode->getTalkerCode();
+                    // TalkerCode object no longer needed.
+                    delete pooledFilterMgr->talkerCode;
+                    pooledFilterMgr->talkerCode = 0;
+                    if (filterMgr->noSBD())
+                        job->sentences = text;
+                    else
+                    {
+                        // Split the text into sentences and store in the job.
+                        // The SBD plugin does all the real sentence parsing, inserting tabs at each
+                        // sentence boundary.
+                        QStringList sentences = QStringList::split("\t", text, false);
+                        int sentenceCount = job->sentences.count();
+                        job->sentences += sentences;
+                        job->partSeqNums.append(sentenceCount + sentences.count());
+                    }
+                    int partNum = job->partSeqNums.count();
+                    // Clean up.
+                    pooledFilterMgr->job = 0;
+                    // Emit signal.
+                    if (!filterMgr->noSBD())
+                    {
+                        if (partNum == 1)
+                            emit textSet(job->appId, job->jobNum);
+                        else
+                            emit textAppended(job->appId, job->jobNum, partNum);
+                    }
+                }
+            }
+        }
+    }
+}
+
+void SpeechData::slotFilterMgrFinished()
+{
+    // kdDebug() << "SpeechData::slotFilterMgrFinished: received signal FilterMgr finished signal." << endl;
+    doFiltering();
+}
+
+void SpeechData::slotFilterMgrStopped()
+{
+    doFiltering();
+}
+
