@@ -32,7 +32,7 @@
 #include <ktoolbar.h>
 #include <kiconloader.h>
 #include <kdebug.h>
-#include <kfiledialog.h>
+#include <kencodingfiledialog.h>
 #include <kapplication.h>
 #include <kinputdialog.h>
 
@@ -90,12 +90,12 @@ KInstance *KttsJobMgrFactory::instance()
 
 KAboutData *KttsJobMgrFactory::aboutData()
 {
-  KAboutData *about = new KAboutData("KttsJobMgr", I18N_NOOP("KttsJobMgr"), "1.99");
+  KAboutData *about = new KAboutData("kttsjobmgr", I18N_NOOP("KttsJobMgr"), "1.99");
   return about;
 }
 
 KttsJobMgrPart::KttsJobMgrPart(QWidget *parent, const char *name) :
-    DCOPStub("kttsd", "kspeech"),
+    DCOPStub("kttsd", "KSpeech"),
     DCOPObject("kttsjobmgr_kspeechsink"),
     KParts::ReadOnlyPart(parent, name)
 {
@@ -186,15 +186,15 @@ KttsJobMgrPart::KttsJobMgrPart(QWidget *parent, const char *name) :
     autoSelectInJobListView();
 
     // Connect DCOP Signals emitted by KTTSD to our own DCOP methods.
-    connectDCOPSignal("kttsd", "kspeech",
+    connectDCOPSignal("kttsd", "KSpeech",
         "kttsdStarted()",
         "kttsdStarted()",
         false);
-    connectDCOPSignal("kttsd", "kspeech",
+    connectDCOPSignal("kttsd", "KSpeech",
         "markerSeen(QCString,QString)",
         "markerSeen(QCString,QString)",
         false);
-    connectDCOPSignal("kttsd", "kspeech",
+    connectDCOPSignal("kttsd", "KSpeech",
         "sentenceStarted(QCString,uint,uint)",
         "sentenceStarted(QCString,uint,uint)",
         false);
@@ -202,31 +202,31 @@ KttsJobMgrPart::KttsJobMgrPart(QWidget *parent, const char *name) :
         "sentenceFinished(QCString,uint,uint)",
         "sentenceFinished(QCString,uint,uint)",
         false);
-    connectDCOPSignal("kttsd", "kspeech",
+    connectDCOPSignal("kttsd", "KSpeech",
         "textSet(QCString,uint)",
         "textSet(QCString,uint)",
         false);
-    connectDCOPSignal("kttsd", "kspeech",
+    connectDCOPSignal("kttsd", "KSpeech",
         "textStarted(QCString,uint)",
         "textStarted(QCString,uint)",
         false);
-    connectDCOPSignal("kttsd", "kspeech",
+    connectDCOPSignal("kttsd", "KSpeech",
         "textFinished(QCString,uint)",
         "textFinished(QCString,uint)",
         false);
-    connectDCOPSignal("kttsd", "kspeech",
+    connectDCOPSignal("kttsd", "KSpeech",
         "textStopped(QCString,uint)",
         "textStopped(QCString,uint)",
         false);
-    connectDCOPSignal("kttsd", "kspeech",
+    connectDCOPSignal("kttsd", "KSpeech",
         "textPaused(QCString,uint)",
         "textPaused(QCString,uint)",
         false);
-    connectDCOPSignal("kttsd", "kspeech",
+    connectDCOPSignal("kttsd", "KSpeech",
         "textResumed(QCString,uint)",
         "textResumed(QCString,uint)",
         false);
-    connectDCOPSignal("kttsd", "kspeech",
+    connectDCOPSignal("kttsd", "KSpeech",
         "textRemoved(QCString,uint)",
         "textRemoved(QCString,uint)",
         false);
@@ -556,7 +556,7 @@ void KttsJobMgrPart::slot_job_change_talker()
         stw->talkersList->setSelectionMode(QListView::Single);
         // A list of the items in the listview.
         QValueList<QListViewItem*> talkersListItems;
-        QListViewItem* talkerItem;
+        QListViewItem* talkerItem = 0;
         // Fill rows and columns.
         uint talkerCodesListCount = talkerCodesList.count();
         for (uint index = 0; index < talkerCodesListCount; index++)
@@ -580,7 +580,7 @@ void KttsJobMgrPart::slot_job_change_talker()
             i18n("Select Talker"),
             KDialogBase::Help|KDialogBase::Ok|KDialogBase::Cancel,
             KDialogBase::Cancel,
-            this->widget(),
+            widget(),
             "selectTalker_dlg",
             true,
             true);
@@ -593,7 +593,7 @@ void KttsJobMgrPart::slot_job_change_talker()
             if (talkerItem)
             {
                 uint index = talkersListItems.findIndex(talkerItem);
-                changeTextTalker(jobNum, talkerCodesList[index]);
+                changeTextTalker(talkerCodesList[index], jobNum);
                 refreshJob(jobNum);
             }
         }
@@ -623,8 +623,14 @@ void KttsJobMgrPart::slot_speak_clipboard()
 
 void KttsJobMgrPart::slot_speak_file()
 {
-    QString filename = KFileDialog::getOpenFileName();
-    setFile(filename, NULL);
+    KEncodingFileDialog dlg;
+    KEncodingFileDialog::Result result = dlg.getOpenFileNameAndEncoding();
+    if (result.fileNames.count() == 1)
+    {
+        // kdDebug() << "KttsJobMgr::slot_speak_file: calling setFile with filename " <<
+        //     result.fileNames[0] << " and encoding " << result.encoding << endl;
+        setFile(result.fileNames[0], NULL, result.encoding);
+    }
 }
 
 void KttsJobMgrPart::slot_refresh()
@@ -652,11 +658,11 @@ QString KttsJobMgrPart::stateToStr(int state)
 {
     switch( state )
     {
-        case kspeech::jsQueued: return        i18n("Queued");
-        case kspeech::jsSpeakable: return     i18n("Waiting");
-        case kspeech::jsSpeaking: return      i18n("Speaking");
-        case kspeech::jsPaused: return        i18n("Paused");
-        case kspeech::jsFinished: return      i18n("Finished");
+        case KSpeech::jsQueued: return        i18n("Queued");
+        case KSpeech::jsSpeakable: return     i18n("Waiting");
+        case KSpeech::jsSpeaking: return      i18n("Speaking");
+        case KSpeech::jsPaused: return        i18n("Paused");
+        case KSpeech::jsFinished: return      i18n("Finished");
         default: return                       i18n("Unknown");
     }
 }
@@ -921,7 +927,7 @@ ASYNC KttsJobMgrPart::sentenceStarted(const QCString&, const uint jobNum, const
     QListViewItem* item = findItemByJobNum(jobNum);
     if (item)
     {
-        item->setText(jlvcState, stateToStr(kspeech::jsSpeaking));
+        item->setText(jlvcState, stateToStr(KSpeech::jsSpeaking));
         item->setText(jlvcPosition, QString::number(seq));
         m_currentSentence->setText(getTextJobSentence(jobNum, seq));
     }
@@ -941,7 +947,7 @@ ASYNC KttsJobMgrPart::sentenceFinished(const QCString&, const uint jobNum, const
     QListViewItem* item = findItemByJobNum(jobNum);
     if (item)
     {
-        item->setText(jlvcState, stateToStr(kspeech::jsSpeaking));
+        item->setText(jlvcState, stateToStr(KSpeech::jsSpeaking));
     }
 */
 }
@@ -1006,7 +1012,7 @@ ASYNC KttsJobMgrPart::textStarted(const QCString&, const uint jobNum)
     QListViewItem* item = findItemByJobNum(jobNum);
     if (item)
     {
-        item->setText(jlvcState, stateToStr(kspeech::jsSpeaking));
+        item->setText(jlvcState, stateToStr(KSpeech::jsSpeaking));
         item->setText(jlvcPosition, "1");
     }
 }
@@ -1026,7 +1032,7 @@ ASYNC KttsJobMgrPart::textFinished(const QCString&, const uint jobNum)
     QListViewItem* item = findItemByJobNum(jobNum);
     if (item)
     {
-        item->setText(jlvcState, stateToStr(kspeech::jsFinished));
+        item->setText(jlvcState, stateToStr(KSpeech::jsFinished));
         // Update sentence pointer, since signal may not be emitted for final CR.
         refreshJob(jobNum);
     }
@@ -1043,7 +1049,7 @@ ASYNC KttsJobMgrPart::textStopped(const QCString&, const uint jobNum)
     QListViewItem* item = findItemByJobNum(jobNum);
     if (item)
     {
-        item->setText(jlvcState, stateToStr(kspeech::jsQueued));
+        item->setText(jlvcState, stateToStr(KSpeech::jsQueued));
         item->setText(jlvcPosition, "1");
     }
 }
@@ -1059,7 +1065,7 @@ ASYNC KttsJobMgrPart::textPaused(const QCString&, const uint jobNum)
     QListViewItem* item = findItemByJobNum(jobNum);
     if (item)
     {
-        item->setText(jlvcState, stateToStr(kspeech::jsPaused));
+        item->setText(jlvcState, stateToStr(KSpeech::jsPaused));
     }
 }
 
@@ -1073,7 +1079,7 @@ ASYNC KttsJobMgrPart::textResumed(const QCString&, const uint jobNum)
     QListViewItem* item = findItemByJobNum(jobNum);
     if (item)
     {
-        item->setText(jlvcState, stateToStr(kspeech::jsSpeaking));
+        item->setText(jlvcState, stateToStr(KSpeech::jsSpeaking));
     }
 }
 
