@@ -1,5 +1,4 @@
 /***************************************************** vim:set ts=4 sw=4 sts=4:
-  festivalintplugin.cpp
   Generating the factories so festival (interactive) can be used as plug in.
   -------------------
   Copyright : (C) 2004 Gary Cramblitt
@@ -22,5 +21,5 @@
 #include "festivalintproc.h"
 
 typedef K_TYPELIST_2( FestivalIntProc, FestivalIntConf ) Festival;
-K_EXPORT_COMPONENT_FACTORY( libkttsd_festivalintplugin, KGenericFactory<Festival>("plugin_FestivalInt") );
+K_EXPORT_COMPONENT_FACTORY( libkttsd_festivalintplugin, KGenericFactory<Festival>("kttsd_festivalintplugin") );
 
